# -*- coding: utf-8 -*-

import matplotlib.pyplot as plt
import plotly.figure_factory as ff
import plotly.graph_objs as go
import plotly
plt.rcParams['font.sans-serif'] = ['SimHei']
plt.rcParams['axes.unicode_minus'] = False


class plot():
    def __init__(self, fig_width, fig_height):
        self.fig_width = fig_width
        self.fig_height = fig_height

    def plot_render(self, data, layout, **kwargs):
        kwargs['output_type'] = 'div'
        plot_str = plotly.offline.plot({"data": data, "layout": layout}, **kwargs)
        print(
            '%%angular <div style="height: %ipx; width: %spx"> %s </div>' % (self.fig_height, self.fig_width, plot_str))

    def plot_table(self, df, title_text=''):
        fig = ff.create_table(df)
        fig.layout.title = title_text
        self.plot_render(fig.data, fig.layout)

    def plot_line(self, df, title_text=''):
        data = []
        for name in list(df.columns):
            trace = go.Scatter(
                x=list(df.index),
                y=df[name].values,
                name=name,
                mode="lines+markers")
            data.append(trace)
        layout = go.Layout(
            title=dict(text=title_text),
            autosize=False, width=self.fig_width, height=self.fig_height,
            yaxis=dict(tickfont=dict(size=12), showgrid=True),
            xaxis=dict(showgrid=True),
            yaxis2={'anchor': 'x', "overlaying": 'y', "side": 'right'},
            template='plotly_white')
        self.plot_render(data, layout)

    def plot_bar(self, df, title_text='', legend_x=0.30):
        data = []
        for name in list(df.columns):
            trace = go.Bar(
                x=list(df.index),
                y=df[name].values,
                name=name)
            data.append(trace)
        layout = go.Layout(
            title=dict(text=title_text),
            autosize=False, width=self.fig_width, height=self.fig_height,
            yaxis=dict(tickfont=dict(size=12), showgrid=True),
            xaxis=dict(showgrid=True),
            legend=dict(orientation="h", x=legend_x),
            template='plotly_white')
        self.plot_render(data, layout)

    def plot_area(self, df, title_text=''):
        data = []
        for name in list(df.columns):
            tmp = go.Scatter(
                x=list(df.index),
                y=df[name].values,
                name=name,
                mode='lines',
                line=dict(width=0.5),
                fill='tonexty',
                stackgroup='one')
            data.append(tmp)
        layout = go.Layout(
            title=title_text,
            autosize=False, width=self.fig_width, height=self.fig_height,
            showlegend=True,
            xaxis=dict(type='category'),
            yaxis=dict(type='linear', range=[1, 100], dtick=20, ticksuffix='%'))
        self.plot_render(data, layout)


from datetime import datetime, timedelta
import pandas as pd
import numpy as np
from hbshare.quant.CChen.cta_factor.factor_func import factor_compute
from hbshare.quant.CChen.cta_factor.factor_algo import (
    tscarry, carry, tsmom, tsrev, xsmom, xsrev, tswr, xswr, mr, mrchg, tsmr,
    tsbasismom, xsbasismom, tspoichg, xspoichg, xssigma,
    xsmr, xstsmr, xsmrchg, tspvolchg, xspvolchg, tsskew, xsskew
)


def run(sql_path, sql_info):
    start_date = datetime(2010, 1, 1).date()
    # end_date = datetime(2021, 4, 9).date()
    end_date = datetime.now().date()

    window_days_list = [1, 2, 3, 5, 10, 20, 30, 40, 50, 60, 90, 100, 150, 200, 250]
    liq_d = 20

    table = 'hsjy_fut_com_index'
    table_wr = 'hsjy_fut_wr'
    table_mr = 'hsjy_fut_memberrank'
    table_contracts_info = 'hsjy_fut_info_c'
    table_raw = 'hsjy_fut_com'

    data = pd.read_sql_query(
        'select * from ' + table + ' where TDATE<='
        + end_date.strftime('%Y%m%d')
        + ' and TDATE>=' + (start_date - timedelta(days=500)).strftime('%Y%m%d')
        + ' order by TDATE',
        sql_path
    )
    data_wr0 = pd.read_sql_query(
        'select TDATE, EXCHANGE, PCODE, PNAME, WRQCURRENT from ' + table_wr + ' where TDATE<='
        + end_date.strftime('%Y%m%d')
        + ' and TDATE>=' + (start_date - timedelta(days=500)).strftime('%Y%m%d')
        + ' and FREQ=5 order by TDATE',
        sql_path
    )
    calendar = data[['TDATE']].sort_values(by='TDATE').drop_duplicates().reset_index(drop=True)
    data_wr = pd.DataFrame()
    for e in data['EXCHANGE'].drop_duplicates().tolist():
        PCODEs = data[data['EXCHANGE'] == e]['PCODE'].drop_duplicates().tolist()
        for p in PCODEs:
            data_wr_p = data_wr0[np.array(data_wr0['EXCHANGE'] == e) & np.array(data_wr0['PCODE'] == p)]
            data_wr_p = pd.merge(calendar, data_wr_p, on='TDATE', how='left').fillna(method='ffill')
            data_wr_p = data_wr_p[data_wr_p['WRQCURRENT'] >= 0].reset_index(drop=True)
            data_wr = pd.concat([data_wr, data_wr_p])
    data_wr = data_wr.reset_index(drop=True)

    data_mr = pd.read_sql_query(
        'select TDATE, CCODE, INDICATORCODE as SIDE, sum(INDICATORVOL) as MR, sum(INDICATORCHG) as MRCHG from ' + table_mr
        + ' where TDATE<=' + end_date.strftime('%Y%m%d')
        + ' and TDATE>=' + (start_date - timedelta(days=500)).strftime('%Y%m%d')
        + ' and INDICATORCODE in (3, 4) GROUP BY INDICATORCODE, TDATE, CCODE order by TDATE, CCODE, INDICATORCODE',
        sql_path
    )
    data_info = pd.read_sql_query(
        'select * from ' + table_contracts_info, sql_path
    )
    data_mr = pd.merge(data_mr, data_info[['CCODE', 'EXCHANGE', 'PCODE']], on='CCODE', how='left')
    data_mr = data_mr.groupby(by=['TDATE', 'SIDE', 'EXCHANGE', 'PCODE']).sum().reset_index()
    data_mr['SIDE'] = data_mr['SIDE'].apply(lambda x: 1 if x == 3 else (-1 if x == 4 else 0))
    data_mr['MR'] = data_mr['MR'] * data_mr['SIDE']
    data_mr['MRCHG'] = data_mr['MRCHG'] * data_mr['SIDE']

    data_contracts = pd.read_sql_query(
        'select * from ' + table_raw + ' where CCODE in ' + str(tuple(np.unique(data[['CCODE', 'CCODE2']].dropna())))
        + ' order by CCODE, TDATE',
        sql_path
    )

    factor_compute(
        window_days_list=[10, 20, 30, 50, 90, 150, 250], hedge_ratio_list=[50, 75], data=data, factor_func=xssigma,
        liq_days=liq_d, data_raw=data_contracts, sql_info=sql_info, to_sql_path=sql_path
    )

    factor_compute(
        window_days_list=window_days_list[:10], hedge_ratio_list=[50, 75], data=data, factor_func=xsbasismom,
        liq_days=liq_d, data_raw=data_contracts, sql_info=sql_info, to_sql_path=sql_path
    )

    factor_compute(
        window_days_list=window_days_list[:10], data=data, factor_func=tsbasismom, liq_days=liq_d,
        data_raw=data_contracts, sql_info=sql_info, to_sql_path=sql_path
    )

    factor_compute(
        window_days_list=[20, 50], hedge_ratio_list=[50, 60, 70, 80, 90], data=data, factor_func=carry, liq_days=liq_d,
        sql_info=sql_info, to_sql_path=sql_path
    )

    factor_compute(
        window_days_list=window_days_list, data=data, factor_func=tsmom, liq_days=liq_d,
        sql_info=sql_info, to_sql_path=sql_path
    )

    factor_compute(
        window_days_list=window_days_list, data=data, factor_func=tscarry, liq_days=liq_d,
        sql_info=sql_info, to_sql_path=sql_path
    )

    factor_compute(
        window_days_list=window_days_list[1:], data=data, factor_func=tsskew, liq_days=liq_d,
        sql_info=sql_info, to_sql_path=sql_path
    )

    factor_compute(
        window_days_list=window_days_list, data=data, factor_func=tsrev, liq_days=liq_d,
        sql_info=sql_info, to_sql_path=sql_path
    )

    factor_compute(
        window_days_list=window_days_list, hedge_ratio_list=[50, 75], data=data, factor_func=xsmom, liq_days=liq_d,
        sql_info=sql_info, to_sql_path=sql_path
    )

    factor_compute(
        window_days_list=window_days_list[1:], hedge_ratio_list=[50, 75], data=data, factor_func=xsskew, liq_days=liq_d,
        sql_info=sql_info, to_sql_path=sql_path
    )

    factor_compute(
        window_days_list=window_days_list, hedge_ratio_list=[50, 75], data=data, factor_func=xsrev, liq_days=liq_d,
        sql_info=sql_info, to_sql_path=sql_path
    )

    factor_compute(
        window_days_list=window_days_list, data=data, factor_func=tspoichg, liq_days=liq_d,
        sql_info=sql_info, to_sql_path=sql_path
    )

    factor_compute(
        window_days_list=window_days_list, data=data, factor_func=tspvolchg, liq_days=liq_d,
        sql_info=sql_info, to_sql_path=sql_path
    )

    factor_compute(
        window_days_list=window_days_list, hedge_ratio_list=[50, 75], data=data, factor_func=xspoichg, liq_days=liq_d,
        sql_info=sql_info, to_sql_path=sql_path
    )

    factor_compute(
        window_days_list=window_days_list, hedge_ratio_list=[50, 75], data=data, factor_func=xspvolchg, liq_days=liq_d,
        sql_info=sql_info, to_sql_path=sql_path
    )

    factor_compute(
        window_days_list=window_days_list,
        hedge_ratio_list=[50, 75],
        data=data,
        data_wr=data_wr,
        factor_func=xswr,
        liq_days=liq_d,
        price_field='OPEN',
        sql_info=sql_info, to_sql_path=sql_path
    )

    factor_compute(
        window_days_list=window_days_list,
        data=data,
        data_wr=data_wr,
        factor_func=tswr,
        liq_days=liq_d,
        price_field='OPEN',
        sql_info=sql_info, to_sql_path=sql_path
    )

    factor_compute(
        window_days_list=window_days_list,
        data=data,
        data_mr=data_mr,
        factor_func=mr,
        liq_days=liq_d,
        price_field='OPEN',
        sql_info=sql_info, to_sql_path=sql_path
    )

    factor_compute(
        window_days_list=window_days_list,
        hedge_ratio_list=[50, 75],
        data=data,
        data_mr=data_mr,
        factor_func=xsmr,
        liq_days=liq_d,
        price_field='OPEN',
        sql_info=sql_info, to_sql_path=sql_path
    )

    factor_compute(
        window_days_list=window_days_list,
        data=data,
        data_mr=data_mr,
        factor_func=mrchg,
        liq_days=liq_d,
        price_field='OPEN',
        sql_info=sql_info, to_sql_path=sql_path
    )

    factor_compute(
        window_days_list=window_days_list,
        hedge_ratio_list=[50, 75],
        data=data,
        data_mr=data_mr,
        factor_func=xsmrchg,
        liq_days=liq_d,
        price_field='OPEN',
        sql_info=sql_info, to_sql_path=sql_path
    )

    factor_compute(
        window_days_list=window_days_list,
        data=data,
        data_mr=data_mr,
        factor_func=tsmr,
        liq_days=liq_d,
        price_field='OPEN',
        sql_info=sql_info, to_sql_path=sql_path
    )

    factor_compute(
        window_days_list=window_days_list,
        hedge_ratio_list=[50, 75],
        data=data,
        data_mr=data_mr,
        factor_func=xstsmr,
        liq_days=liq_d,
        price_field='OPEN',
        sql_info=sql_info, to_sql_path=sql_path
    )





# -*- coding: utf-8 -*-

from datetime import datetime, timedelta
import numpy as np
import pandas as pd

import matplotlib.pyplot as plt
plt.rcParams['font.sans-serif'] = ['SimHei']
plt.rcParams['axes.unicode_minus'] = False


def get_downward_volatility(idxs, data):
    """
    idxs: 计算数据索引
    data: 所需数据
    """
    lambdaa = 0.94
    part_df = data.iloc[list(map(int, idxs))].copy(deep=True)
    L = len(part_df)
    part_df['IDX_'] = range(L, 0, -1)
    part_df['LAMBDAA_ADJUST'] = lambdaa ** (part_df['IDX_'] - 1)
    part_df_downward = part_df[part_df['RET'] < 0]
    N = len(part_df_downward)
    sigma_2 = ((1 - lambdaa) * (part_df_downward['LAMBDAA_ADJUST'] * part_df_downward['RET'] ** 2).sum() + lambdaa ** L * (part_df_downward['RET'] ** 2).sum() / float(N)) / ((1 - lambdaa) * part_df_downward['LAMBDAA_ADJUST'].sum() + lambdaa ** L)
    sigma = np.sqrt(sigma_2)
    return sigma

def get_equity_weight(sigma, sigma_star, equity_uplimit):
    """
    sigma：EWMA下行波动率，float
    sigma_star：波动率风险偏好，float
    equity_uplimit：权益上限，float
    """
    equity_weight = max(min(sigma_star / (np.sqrt(240) * sigma), equity_uplimit), 0)
    return equity_weight

def get_dvol_and_eweight_single(index, StartDate, EndDate, N, sigma_star, equity_uplimit):
    """
    index：资产代码，str，例如'000300'
    StartDate：起始日期, str, 例如‘20190110’
    EndDate：结束日期, str, 例如‘20220401’
    N：回溯期间，即用过去N个交易日数据计算，int，例如20
    sigma_star：波动率风险偏好，float，例如0.02
    equity_uplimit：权益上限，float，例如0.80
    """
    # 由于计算StartDate当日EWMA下行波动率需要往前回溯N个交易日数据，所以取数据要比StartDate往前多取一段时间，这里取N * 2
    data_StartDate = (datetime.strptime(StartDate, '%Y%m%d') - timedelta(N * 2)).strftime('%Y%m%d')

    ####################################################################################################################
    # 需要变更为通过api取index的资产价格
    # 例如 index_daily_k = pro.getDataAPI('index', data_StartDate, EndDate)，注意取数据起始时间是data_StartDate
    # 整理成一共四列数据：['INDEX_NAME', 'INDEX_CODE', 'TRADE_DATE', 'CLOSE_INDEX']，分别代表资产名称, 资产代码, 交易日期, 资产价格，并按照交易日期升序排序
    index_daily_k = pd.read_excel('D:/资产价格.xlsx', index_col=0)
    index_daily_k['资产名称'] = index_daily_k['资产名称'].astype(str)
    index_daily_k['资产代码'] = index_daily_k['资产代码'].apply(lambda x: str(x).zfill(6))
    index_daily_k['交易日期'] = index_daily_k['交易日期'].astype(str)
    index_daily_k['资产价格'] = index_daily_k['资产价格'].astype(float)
    index_daily_k = index_daily_k.sort_values('交易日期')
    index_daily_k.columns = ['INDEX_NAME', 'INDEX_CODE', 'TRADE_DATE', 'CLOSE_INDEX']
    index_daily_k = index_daily_k[(index_daily_k['TRADE_DATE'] >= data_StartDate) & (index_daily_k['TRADE_DATE'] <= EndDate)]
    ####################################################################################################################

    index_daily_k['CLOSE_INDEX'] = index_daily_k['CLOSE_INDEX'].replace(0.0, np.nan).fillna(method='ffill')
    index_daily_k['RET'] = np.log(index_daily_k['CLOSE_INDEX'] / index_daily_k['CLOSE_INDEX'].shift())
    index_daily_k['RET'] = index_daily_k['RET'].shift()  # 计算t日的下行波动率用【t-1, t-2, ... , t-20】的对数收益率数据，所以数据需要shift一下
    index_daily_k = index_daily_k.dropna(subset=['RET'])
    index_daily_k = index_daily_k[['INDEX_NAME', 'INDEX_CODE', 'TRADE_DATE', 'CLOSE_INDEX', 'RET']]
    index_daily_k['IDX'] = range(len(index_daily_k.index))
    index_daily_k['SIGMA'] = index_daily_k['IDX'].rolling(window=N, min_periods=N, center=False).apply(lambda x: get_downward_volatility(x, index_daily_k))
    index_daily_k['EUQITY_WEIGHT'] = index_daily_k['SIGMA'].apply(lambda x: get_equity_weight(x, sigma_star, equity_uplimit))
    index_daily_k = index_daily_k[(index_daily_k['TRADE_DATE'] >= StartDate) & (index_daily_k['TRADE_DATE'] <= EndDate)]
    result = index_daily_k[['INDEX_NAME', 'INDEX_CODE', 'TRADE_DATE', 'CLOSE_INDEX', 'RET', 'SIGMA', 'EUQITY_WEIGHT']]
    result.columns = ['资产名称', '资产代码', '交易日期', '资产价格', '对数收益率', 'EWMA下行波动率', '权益资产仓位']

    ####################################################################################################################
    save_path = 'D:/'  # 需要修改成自己的存储路径
    ####################################################################################################################
    result.to_excel('{0}下行波动率.xlsx'.format(save_path))
    fig, ax = plt.subplots(4, 1, figsize=(6, 12))
    pic_list = ['资产价格', '对数收益率', 'EWMA下行波动率', '权益资产仓位']
    result['交易日期'] = result['交易日期'].apply(lambda x: datetime.strptime(x, '%Y%m%d').date())
    for i, pic_name in enumerate(pic_list):
        ax[i].plot(result['交易日期'].values, result[pic_name].values)
        ax[i].set_title('{0}'.format(pic_name))
        ax[i].set_xticklabels(labels=result['交易日期'], rotation=90)
    plt.tight_layout()
    plt.savefig('{0}result.png'.format(save_path))
    return result

if __name__ == '__main__':
    index = '000300'  # 资产代码
    StartDate = '20190110'  # 起始日期
    EndDate = '20220401'  # 结束日期
    N = 20  # 回溯期间，即用过去N个交易日数据计算
    sigma_star = 0.02  # 波动率风险偏好
    equity_uplimit = 0.8  # 权益上限
    dvol_and_eweight_single = get_dvol_and_eweight_single(index=index, StartDate=StartDate, EndDate=EndDate, N=N,
                                                          sigma_star=sigma_star, equity_uplimit=equity_uplimit)


"""
tracker 用于季度回顾/半年度回顾等场景
"""
import numpy as np
import pandas as pd
import hbshare as hbs
from datetime import datetime
from hbshare.quant.Kevin.quant_room.MyUtil.data_loader import get_fund_nav_from_sql, get_trading_day_list


class FundTracker:
    def __init__(self, start_date, end_date):
        self.start_date = start_date
        self.end_date = end_date
        self._load_data()

    @staticmethod
    def _load_benchmark(start_date, end_date, benchmark_id):
        sql_script = "SELECT JYRQ as TRADEDATE, ZQDM, SPJG as TCLOSE from funddb.ZSJY WHERE" \
                     " ZQDM = '{}' " \
                     "and JYRQ >= {} and JYRQ <= {}".format(benchmark_id, start_date, end_date)
        res = hbs.db_data_query('readonly', sql_script, page_size=5000)
        data = pd.DataFrame(res['data'])
        benchmark_df = data.set_index('TRADEDATE')['TCLOSE']

        return benchmark_df

    def _load_data(self):
        trading_day_list = get_trading_day_list(self.start_date, self.end_date, frequency="week")

        # 500指增
        config_df = pd.read_excel('D:\\量化产品跟踪\\tracker\\config.xlsx', sheet_name=2)
        fund_dict = config_df.set_index('管理人')['fund_id'].to_dict()

        fund_nav = get_fund_nav_from_sql(self.start_date, self.end_date, fund_dict).reindex(
            trading_day_list).fillna(method='bfill')
        benchmark_series = self._load_benchmark(self.start_date, self.end_date, '000905').reindex(
            trading_day_list).pct_change().dropna()
        return_df = fund_nav.pct_change().dropna().sub(benchmark_series, axis=0).sort_index()
        # 修正
        return_df.loc[:"20210806", '伯兄'] = np.NAN
        return_df.loc[:"20210618", '概率'] = np.NAN
        return_df.loc[:"20211231", '白鹭'] = np.NAN

        nav_df = (1 + return_df).cumprod()
        nav_df.loc["20211231", "白鹭"] = 1.

        # 1000指增
        # config_df = pd.read_excel('D:\\量化产品跟踪\\tracker\\config.xlsx', sheet_name=0)
        # fund_dict = config_df[config_df['level2type'] == '1000指增'].set_index('fund_name')['fund_id'].to_dict()
        #
        # fund_nav = get_fund_nav_from_sql(self.start_date, self.end_date, fund_dict).reindex(
        #     trading_day_list).fillna(method='bfill')
        # benchmark_series = self._load_benchmark(self.start_date, self.end_date, '000852').reindex(
        #     trading_day_list).pct_change().dropna()
        # return_df = fund_nav.pct_change().dropna().sub(benchmark_series, axis=0).sort_index()
        # return_df.columns = [x[:2] for x in return_df.columns]
        # # return_df.loc[:"20211008", '启林'] = np.NAN
        # # return_df.loc[:"20220107", '伯兄'] = np.NAN
        #
        # nav_df = (1 + return_df).cumprod()

        # 月度收益
        nav_df['trade_date'] = nav_df.index
        nav_df['trade_dt'] = nav_df['trade_date'].apply(lambda x: datetime.strptime(x, "%Y%m%d"))
        nav_df['month'] = nav_df['trade_dt'].apply(lambda x: x.month)
        nav_df['year'] = nav_df['trade_dt'].apply(lambda x: x.year)
        nav_df['date'] = nav_df['year'].map(str) + '.' + nav_df['month'].map(str)
        month_end = nav_df[nav_df['month'].shift(-1) != nav_df['month']]['trade_date'].tolist()

        month_excess = nav_df.reindex(month_end)[return_df.columns].pct_change().dropna()
        month_excess = pd.merge(month_excess, nav_df[['date']], left_index=True, right_index=True)
        month_excess = month_excess.set_index('date').T

        # 分标签分析
        return_df = pd.merge(
            return_df.T, config_df.set_index('管理人')[['规模', '策略频率', '风格敞口']], left_index=True, right_index=True)

        writer = pd.ExcelWriter("D:\\量化产品跟踪\\tracker\\分标签净值.xlsx", engine="xlsxwriter")

        # 规模分析
        tmp = return_df.groupby('规模')[return_df.columns].mean().T.sort_index()
        nav_df = (1 + tmp).cumprod()
        cols = nav_df.columns.tolist()
        nav_df['trade_date'] = nav_df.index
        nav_df = nav_df.reset_index(drop=True)
        nav_df = nav_df[['trade_date'] + cols]
        nav_df.to_excel(writer, sheet_name='规模分类', index=False)

        # 频段分析
        tmp = return_df.groupby('策略频率')[return_df.columns].mean().T.sort_index()
        nav_df = (1 + tmp).cumprod()
        cols = nav_df.columns.tolist()
        nav_df['trade_date'] = nav_df.index
        nav_df = nav_df.reset_index(drop=True)
        nav_df = nav_df[['trade_date'] + cols]
        nav_df.to_excel(writer, sheet_name='频率分类', index=False)

        # 敞口分析
        tmp = return_df.groupby('风格敞口')[return_df.columns].mean().T.sort_index()
        nav_df = (1 + tmp).cumprod()
        cols = nav_df.columns.tolist()
        nav_df['trade_date'] = nav_df.index
        nav_df = nav_df.reset_index(drop=True)
        nav_df = nav_df[['trade_date'] + cols]
        nav_df.to_excel(writer, sheet_name='风格敞口分类', index=False)

        writer.save()
        writer.close()


if __name__ == '__main__':
    # from hbshare.quant.CChen.output import index_fut_basis_wind
    #
    # s_date = datetime.strptime('20170101', '%Y%m%d')
    # e_date = datetime.strptime('20220812', '%Y%m%d')
    #
    # index_fut_basis_wind('IC', s_date, e_date)

    FundTracker('20210520', '20220729').run()


import pandas as pd
import numpy as np
from datetime import datetime
import hbshare as hbs
from sqlalchemy import create_engine
from hbshare.quant.Kevin.rm_associated.config import engine_params
from hbshare.fe.common.util.data_loader import get_trading_day_list
import statsmodels.api as sm
import plotly
import plotly.graph_objs as go
from plotly.offline import plot as plot_ly


class IndexPremium:
    def __init__(self, start_date, end_date):
        self.start_date = start_date
        self.end_date = end_date
        self._load_data()

    def _load_data(self):
        sql_script = "SELECT * FROM mac_stock_pe_ttm where TRADE_DATE >= {} and TRADE_DATE <= {}".format(
            self.start_date, self.end_date)
        engine = create_engine(engine_params)
        data = pd.read_sql(sql_script, engine)
        data['trade_date'] = data['trade_date'].apply(lambda x: datetime.strftime(x, '%Y%m%d'))
        pe_data = data.set_index('trade_date')['ZZ500']

        sql_script = "SELECT JYRQ as TRADEDATE, ZQDM, SPJG as TCLOSE from funddb.ZSJY WHERE" \
                     " ZQDM = '{}' " \
                     "and JYRQ >= {} and JYRQ <= {}".format('000905', self.start_date, self.end_date)
        res = hbs.db_data_query('readonly', sql_script, page_size=5000)
        data = pd.DataFrame(res['data'])
        benchmark_df = data.set_index('TRADEDATE')['TCLOSE']

        df = pd.merge(pe_data.to_frame('pe_ttm'), benchmark_df.to_frame('index'), left_index=True, right_index=True)
        df['future_ret'] = df['index'].shift(-252) / df['index'] - 1
        trading_day_list = get_trading_day_list(self.start_date, self.end_date, frequency='week')
        df = df.reindex(trading_day_list).dropna()
        df['trade_date'] = df.index
        df['year'] = df['trade_date'].apply(lambda x: datetime.strptime(x, "%Y%m%d").year)
        self.market_df = df[['pe_ttm', 'future_ret', 'year']]

    def run(self):
        market_df = self.market_df

        # func = lambda x: pd.Series(x).rank(pct=True).iloc[-1]
        # n = 5 * 52
        # rolling_pct = market_df['pe_ttm'].rolling(window=n, center=False, min_periods=n).apply(func)
        # market_df['pe_ttm'] = rolling_pct
        # market_df = market_df[market_df.index >= '20100101'].dropna()

        data = []
        for year in sorted(market_df['year'].unique()):
            period_df = market_df[market_df['year'] == year].sort_values(by='pe_ttm')
            trace = go.Scatter(
                x=period_df['pe_ttm'].tolist(),
                y=period_df['future_ret'],
                name='{}'.format(year),
                mode='markers'
            )
            data.append(trace)

        layout = go.Layout(
            title=dict(text="中证500估值 VS 持有一年收益（分年度）"),
            autosize=False, width=1200, height=800,
            yaxis=dict(tickfont=dict(size=12), tickformat=',.1%'),
            xaxis=dict(showgrid=True),
            template='plotly_white'
        )

        # regression
        model = sm.OLS(market_df['future_ret'], sm.add_constant(market_df['pe_ttm'])).fit()
        const, beta = model.params[0], model.params[1]

        t = np.linspace(market_df['pe_ttm'].min(), market_df['pe_ttm'].max(), 100)

        trace0 = go.Scatter(
            x=t,
            y=const + beta * t,
            name="拟合线",
            mode='lines'
        )
        data.append(trace0)

        figure = go.Figure(data=data, layout=layout)
        plot_ly(figure, filename='D:\\123.html', auto_open=False)

#
#     date_list = df.index.tolist()
#     n = int(len(date_list) / 12)
#     tick_vals = [i for i in range(0, len(df), n)]
#     tick_text = [date_list[i] for i in range(0, len(df), n)]
#
#     layout = go.Layout(
#         title=dict(text="中证500估值和持有一年收益走势图"),
#         autosize=False, width=1200, height=600,
#         yaxis=dict(tickfont=dict(size=12), showgrid=False),
#         yaxis2=dict(tickfont=dict(size=12), tickformat=',.1%', overlaying='y', side="right"),
#         xaxis=dict(showgrid=True, tickvals=tick_vals, ticktext=tick_text),
#         legend=dict(orientation="h", x=0.4),
#         template='plotly_white')
#
#     figure = go.Figure(data=data, layout=layout)
#     plot_ly(figure, filename='D:\\123.html', auto_open=False)


if __name__ == '__main__':
    IndexPremium('20050101', '20220419').run()


"""
A股市场月末的概念标签
"""
from WindPy import w
import pandas as pd
import hbshare as hbs
from datetime import datetime, timedelta
from hbshare.quant.Kevin.asset_allocation.macro_index.util import create_table, delete_duplicate_records, WriteToDB

w.start()

selected_concept = ['顺周期', '资源股', '半导体产业', '光伏', '锂电池', '大消费', '元宇宙', '新基建']


class StockConception:
    def __init__(self, start_date, end_date, is_increment=1):
        self.start_date = start_date
        self.end_date = end_date
        self.is_increment = is_increment
        self.table_name = "stock_conception"
        self._load_calendar()

    def _load_calendar(self):
        start_date = self.start_date.replace('-', '')
        end_date = self.end_date.replace('-', '')
        sql_script = "SELECT JYRQ, SFJJ, SFZM, SFYM FROM funddb.JYRL WHERE JYRQ >= {} and JYRQ <= {}".format(
            start_date, end_date)
        res = hbs.db_data_query('readonly', sql_script, page_size=5000)
        df = pd.DataFrame(res['data']).rename(
            columns={"JYRQ": 'calendarDate', "SFJJ": 'isOpen',
                     "SFZM": "isWeekEnd", "SFYM": "isMonthEnd"}).sort_values(by='calendarDate')
        df['isOpen'] = df['isOpen'].astype(int).replace({0: 1, 1: 0})
        df['isWeekEnd'] = df['isWeekEnd'].fillna(0).astype(int)
        df['isMonthEnd'] = df['isMonthEnd'].fillna(0).astype(int)

        self.calendar_df = df

    def get_stock_concept_data(self):
        trading_day_list = self.calendar_df[self.calendar_df['isMonthEnd'] == 1]['calendarDate'].tolist()
        all_concept = []
        for date in trading_day_list:
            # 股票池
            sql_script = "SELECT SYMBOL, TDATE, VATURNOVER, TCAP FROM finchina.CHDQUOTE WHERE TDATE = {}".format(date)
            data = pd.DataFrame(hbs.db_data_query('readonly', sql_script, page_size=5000)['data'])
            data = data[data['SYMBOL'].str[0].isin(['0', '3', '6'])]
            data = data[data['VATURNOVER'] > 1e-6]
            data['sec_code'] = data['SYMBOL'].apply(lambda x: x + '.SH' if x[0] == '6' else x + '.SZ')
            ticker_list = data['sec_code'].tolist()
            # 概念数据
            res = w.wss(','.join(ticker_list), 'concept', "tradeDate={}".format(date))
            if res.ErrorCode != 0:
                df = pd.DataFrame()
                print("fetch concept data error: trade_date = {}".format(date))
            else:
                df = pd.DataFrame(res.Data[0], index=res.Codes, columns=res.Fields)
                df.index.name = 'sec_code'
            # process
            concept_list = []
            for sec in df.index.tolist():
                sub_list = df.loc[sec, 'CONCEPT'].split(';')
                concept_list = list(set(concept_list).union(set(sub_list)))

            concept_df = pd.DataFrame(index=df.index, columns=concept_list)
            for sec in concept_df.index.tolist():
                sub_list = df.loc[sec, 'CONCEPT'].split(';')
                concept_df.loc[sec, sub_list] = 1

            concept_df = concept_df[selected_concept].dropna(how='all')
            concept_df.reset_index(inplace=True)
            concept_df['trade_date'] = date
            concept_df['ticker'] = concept_df['sec_code'].apply(lambda x: x.split('.')[0])
            concept_df = concept_df[['trade_date', 'ticker'] + selected_concept]
            concept_df = pd.melt(concept_df, id_vars=['trade_date', 'ticker'],
                                 value_vars=selected_concept, var_name='concept').dropna()
            concept_df = concept_df[['trade_date', 'ticker', 'concept']]

            all_concept.append(concept_df)

        all_concept = pd.concat(all_concept)

        return all_concept

    def get_construct_result(self):
        if self.is_increment == 1:
            data = self.get_stock_concept_data()
            sql_script = "delete from {} where trade_date in ({})".format(
                self.table_name, ','.join(data['trade_date'].tolist()))
            # delete first
            delete_duplicate_records(sql_script)
            # add new records
            WriteToDB().write_to_db(data, self.table_name)
        else:
            sql_script = """
                create table stock_conception(
                id int auto_increment primary key,
                trade_date date not null,
                ticker varchar(10),
                concept varchar(10)) 
            """
            create_table(self.table_name, sql_script)
            data = self.get_stock_concept_data()
            WriteToDB().write_to_db(data, self.table_name)


if __name__ == '__main__':
    StockConception('2021-01-01', '2022-01-31', is_increment=0).get_construct_result()


#!/usr/bin/python
#coding:utf-8
import pandas as pd
from sqlalchemy import create_engine
from datetime import datetime


# 数据库参数配置
engine = create_engine("mysql+pymysql://{}:{}@{}:{}/{}".format(
    'admin', 'mysql', '192.168.223.152', '3306', 'riskmodel'))

# 经济宏观数据入库
data = pd.read_excel('D:\\kevin\\数据汇总.xlsx', sheet_name=0).rename(
    columns={"日期": "trade_date", "经济增长": "economy_increase"})
data['trade_date'] = data['trade_date'].apply(lambda x: datetime.strftime(x, "%Y%m%d"))
data.to_sql('macro_economy', engine, index=False, if_exists='append')

# 市场宏观数据入库
data = pd.read_excel('D:\\kevin\\数据汇总.xlsx', sheet_name=1).rename(
    columns={"日期": "trade_date", "万德全A": "wind_A", "沪深300": "HS300", "中证500": "ZZ500"})
data['trade_date'] = data['trade_date'].apply(lambda x: datetime.strftime(x, "%Y%m%d"))
data.to_sql('market_macro', engine, index=False, if_exists='append')

# 市场微观数据入库
data1 = pd.read_excel("D:\\kevin\\数据汇总.xlsx", sheet_name=2).rename(
    columns={"日期": "trade_date", "万德全A": "wind_A", "沪深300": "HS300", "中证500": "ZZ500"})
data1['trade_date'] = data1['trade_date'].apply(lambda x: datetime.strftime(x, "%Y%m%d"))
data1['direction'] = 'left'
data2 = pd.read_excel("D:\\kevin\\数据汇总.xlsx", sheet_name=3).rename(
    columns={"日期": "trade_date", "万德全A": "wind_A", "沪深300": "HS300", "中证500": "ZZ500"})
data2['trade_date'] = data2['trade_date'].apply(lambda x: datetime.strftime(x, "%Y%m%d"))
data2['direction'] = 'right'
data = pd.concat([data1, data2])
data.to_sql('market_micro', engine, index=False, if_exists='append')



composite_factor = {
    'tsmom_dzq': [
        'tsmom_d3',
        'tsmom_d5'
    ],
    'tsmom_zzq': [
        'tsmom_d10',
        'tsmom_d20',
        'tsmom_d30'
    ],
    'tsmom_czq': [
        'tsmom_d60',
        'tsmom_d250',
    ],
    'xsmom_dzq': [
        'xsmom_d3_q75',
        'xsmom_d5_q75'
    ],
    'xsmom_zzq': [
        'xsmom_d10_q75',
        'xsmom_d20_q75',
        'xsmom_d30_q75'
    ],
    'xsmom_czq': [
        'xsmom_d60_q75',
        'xsmom_d250_q75',
    ],
    'qxjg': [
        'carry_d20_q70',
        'tscarry_d50',
    ],
    'tsbasismom_dzq': [
        'tsbasismom_d3',
        'tsbasismom_d5'
    ],
    'tsbasismom_zzq': [
        'tsbasismom_d10',
        'tsbasismom_d20',
        'tsbasismom_d30'
    ],
}



"""
指增策略 - 市场结构
"""
import pandas as pd
import hbshare as hbs
import numpy as np
from scipy import stats
import statsmodels.api as sm
from sqlalchemy import create_engine
from hbshare.quant.Kevin.rm_associated.config import engine_params, style_names
import datetime
import plotly.graph_objs as go
from plotly.offline import plot as plot_ly
from statsmodels.stats.weightstats import DescrStatsW
from hbshare.fe.common.util.data_loader import get_trading_day_list


class MarketHist:
    """
    市场收益率Histogram
    """
    def __init__(self, trade_date, benchmark_id):
        self.trade_date = trade_date
        self.benchmark_id = benchmark_id
        self._load_data()

    def _load_data(self):
        # A股行情
        sql_script = "SELECT SYMBOL, SNAME, VOTURNOVER, PCHG, MCAP, TCAP FROM finchina.CHDQUOTE WHERE" \
                     " TDATE = {}".format(self.trade_date)
        data = pd.DataFrame(hbs.db_data_query('readonly', sql_script, page_size=5000)['data'])
        data = data[data['SYMBOL'].str[0].isin(['0', '3', '6'])]
        data = data[data['VOTURNOVER'] > 1e-8]
        data = data[~data['SNAME'].str.contains('ST')]
        data = data[~data['SNAME'].str.contains('N')]
        data = data[~data['SNAME'].str.contains('C')]
        market_df = data.rename(
            columns={"SYMBOL": "ticker", 'PCHG': "return", "TCAP": "marketValue"})[
            ['ticker', 'return', 'marketValue']].dropna()
        # 指数收益率
        start_dt = datetime.datetime.strptime(self.trade_date, '%Y%m%d')
        pre_date = (start_dt - datetime.timedelta(days=30)).strftime('%Y%m%d')
        sql_script = "SELECT JYRQ as TRADEDATE, ZQMC as INDEXNAME, SPJG as TCLOSE from funddb.ZSJY WHERE ZQDM = '{}' " \
                     "and JYRQ >= {} and JYRQ <= {}".format(self.benchmark_id, pre_date, self.trade_date)
        data = pd.DataFrame(hbs.db_data_query('readonly', sql_script)['data'])
        data['index_return'] = data['TCLOSE'].pct_change()
        index_return = data.set_index('TRADEDATE').loc[self.trade_date, 'index_return']

        self.market_df = market_df
        self.index_return = index_return

    def daily_plot(self):
        market_df = self.market_df.copy()
        market_df = market_df[market_df['return'].abs() <= 10]
        # bins
        interval_range = pd.interval_range(-10, 10, 100, closed='left')
        count_df = pd.cut(market_df['return'], bins=interval_range).value_counts().sort_index().to_frame('num')
        count_df['num_cumsum'] = count_df['num'].cumsum()
        count_df['start'] = count_df.index
        count_df['start'] = count_df['start'].apply(lambda x: round(x.left, 1))
        count_df['color'] = '#55CBF2'
        # locate the index return
        index_return = self.index_return * 100.
        lower = round(np.floor(index_return / 0.2) * 0.2, 2)
        count_df.loc[count_df['start'] == lower, 'color'] = '#ff2d51'
        # quantile
        win_ratio = round(100 - stats.percentileofscore(np.array(market_df['return']), index_return), 1)
        win_med = market_df[market_df['return'] >= index_return]['return'].median()
        lose_med = market_df[market_df['return'] < index_return]['return'].median()
        # locate the lower med
        lower = round(np.floor(lose_med / 0.2) * 0.2, 2)
        count_df.loc[count_df['start'] == lower, 'color'] = '#ffa400'
        # # locate the upper med
        lower = round(np.floor(win_med / 0.2) * 0.2, 2)
        count_df.loc[count_df['start'] == lower, 'color'] = '#ffa400'

        color_list = count_df[count_df['num_cumsum'] > 0]['color'].tolist()

        # text
        map_dict = {"000905": "中证500", "000852": "中证1000", "000300": "沪深300"}
        title_text = "【{}】市场个股收益率结构, {}收益率{}%, 跑赢指数的个股比例为{}%, 跑赢的中位数为{}%, 跑输的中位数为{}%".format(
            self.trade_date, map_dict[self.benchmark_id], round(index_return, 2),
            win_ratio, round(win_med, 2), round(lose_med, 2))

        L = go.Layout(xaxis=dict(range=[-10, 10]), title=title_text, width=1440, height=800)
        D = go.Histogram(x=np.array(market_df['return']), xbins=dict(start=-10, end=10, size=0.2),
                         histfunc='count', histnorm='probability',
                         marker=dict(color=color_list, line=dict(color='white', width=1)))
        F = go.Figure(data=[D], layout=L)
        plot_ly(F, filename='D:\\市场微观结构图\\个股收益率结构\\{}.html'.format(self.trade_date), auto_open=False)


class AlphaSeries:
    def __init__(self, start_date, end_date, benchmark_id='000905'):
        self.start_date = start_date
        self.end_date = end_date
        self.benchmark_id = benchmark_id

    def calculate(self):
        nav_series_list = []
        for name, fund_id in alpha_dict.items():
            sql_script = "SELECT a.jjdm fund_id, b.jzrq TRADEDATE, b.fqdwjz as ADJ_NAV from " \
                         "st_hedge.t_st_jjxx a, st_hedge.t_st_rhb b where a.cpfl = '4' and a.jjdm = b.jjdm " \
                         "and a.jjzt not in ('3') " \
                         "and a.jjdm = '{}' and b.jzrq >= {} and b.jzrq <= {} " \
                         "order by b.jzrq".format(fund_id, self.start_date, self.end_date)
            res = hbs.db_data_query("highuser", sql_script, page_size=5000)
            data = pd.DataFrame(res['data']).set_index('TRADEDATE')['ADJ_NAV']
            data.name = name
            nav_series_list.append(data)
        nav_df = pd.concat(nav_series_list, axis=1).sort_index()
        # 衍复前期的剔除掉
        nav_df.loc[:"20200219", "衍复指增三号"] = np.NaN
        # 处理: 单独只有衍复有的去掉
        nav_df = nav_df.dropna(subset=nav_df.columns[1:], how='all', axis=0)

        trading_day_list = get_trading_day_list(nav_df.index[0], nav_df.index[-1])
        nav_df = nav_df.reindex(trading_day_list).dropna(how='all', axis=0)

        for col in nav_df.columns:
            nav_df[col] = nav_df[col] / nav_df[col].shift(1) - 1
        nav_df.dropna(axis=0, how='all', inplace=True)
        mean_alpha = nav_df.mean(axis=1)  # 策略日度平均收益

        sql_script = "SELECT JYRQ as TRADEDATE, ZQMC as INDEXNAME, SPJG as TCLOSE from funddb.ZSJY WHERE ZQDM = '{}' " \
                     "and JYRQ >= {} and JYRQ <= {}".format(self.benchmark_id, self.start_date, self.end_date)
        data = pd.DataFrame(hbs.db_data_query('readonly', sql_script)['data'])
        data['index_return'] = data['TCLOSE'].pct_change()
        index_return = data.set_index('TRADEDATE')['index_return'].dropna()

        idx = mean_alpha.index.intersection(index_return.index)
        excess_return = mean_alpha.reindex(idx) - index_return.reindex(idx)
        # excess_return = mean_alpha.reindex(idx)

        return excess_return


def get_market_turnover(start_date, end_date):
    """
    市场成交额数据
    """
    sql_script = "SELECT trade_date, amt_sh, amt_sz, amt_300, amt_500, amt_1000 FROM mac_stock_trading where " \
                 "trade_date >= {} and trade_date <= {}".format(start_date, end_date)
    engine = create_engine(engine_params)
    data = pd.read_sql(sql_script, engine)
    data['trade_date'] = data['trade_date'].apply(lambda x: datetime.datetime.strftime(x, '%Y%m%d'))
    data['amt_A'] = data['amt_sh'] + data['amt_sz']
    data['amt_other'] = data['amt_A'] - data['amt_300'] - data['amt_500'] - data['amt_1000']

    return data.set_index('trade_date')[['amt_A', 'amt_300', 'amt_500', 'amt_1000', 'amt_other']]


def get_style_factor(start_date, end_date):
    """
    风格因子收益率
    """
    trade_dt = datetime.datetime.strptime(start_date, '%Y%m%d')
    pre_date = (trade_dt - datetime.timedelta(days=30)).strftime('%Y%m%d')
    sql_script = "SELECT * FROM factor_return where trade_date >= {} and trade_date <= {}".format(pre_date, end_date)
    engine = create_engine(engine_params)
    data = pd.read_sql(sql_script, engine)
    data['trade_date'] = data['trade_date'].apply(lambda x: datetime.datetime.strftime(x, '%Y%m%d'))
    data = pd.pivot_table(
        data, index='trade_date', columns='factor_name', values='factor_ret').sort_index()[style_names]

    return data


class MarketSpread:
    def __init__(self, trade_date):
        self.trade_date = trade_date
        self._load_data()

    @staticmethod
    def _load_shift_date(date):
        trade_dt = datetime.datetime.strptime(date, '%Y%m%d')
        pre_date = (trade_dt - datetime.timedelta(days=100)).strftime('%Y%m%d')

        sql_script = "SELECT JYRQ, SFJJ, SFZM, SFYM FROM funddb.JYRL WHERE JYRQ >= {} and JYRQ <= {}".format(
            pre_date, date)
        res = hbs.db_data_query('readonly', sql_script, page_size=5000)
        df = pd.DataFrame(res['data']).rename(
            columns={"JYRQ": 'calendarDate', "SFJJ": 'isOpen',
                     "SFZM": "isWeekEnd", "SFYM": "isMonthEnd"}).sort_values(by='calendarDate')
        df['isOpen'] = df['isOpen'].astype(int).replace({0: 1, 1: 0})
        df['isWeekEnd'] = df['isWeekEnd'].fillna(0).astype(int)
        df['isMonthEnd'] = df['isMonthEnd'].fillna(0).astype(int)

        trading_day_list = df[df['isMonthEnd'] == 1]['calendarDate'].tolist()

        return trading_day_list[-1]

    @staticmethod
    def _load_benchmark_components(date):
        sql_script = "SELECT * FROM hsjy_gg.SecuMain where SecuCategory = 4 and " \
                     "SecuCode in ('000300', '000905', '000852')"
        res = hbs.db_data_query('readonly', sql_script)
        index_info = pd.DataFrame(res['data'])
        inner_code_series = index_info.set_index('SECUCODE')['INNERCODE']

        weight = []
        for benchmark_id in ['000300', '000905', '000852']:
            inner_code = inner_code_series.loc[benchmark_id]
            sql_script = "SELECT (select a.SecuCode from hsjy_gg.SecuMain a where a.InnerCode = b.InnerCode and " \
                         "rownum = 1) SecuCode, b.EndDate, b.Weight FROM hsjy_gg.LC_IndexComponentsWeight b WHERE " \
                         "b.IndexCode = '{}' and b.EndDate = to_date('{}', 'yyyymmdd')".format(inner_code, date)
            data = pd.DataFrame(hbs.db_data_query('readonly', sql_script)['data'])
            weight_df = data.rename(
                columns={"SECUCODE": "ticker", "ENDDATE": "effDate", "WEIGHT": "weight"})
            weight_df['benchmark_id'] = benchmark_id
            weight.append(weight_df[['ticker', 'benchmark_id']])

        return pd.concat(weight)

    def _load_data(self):
        # A股行情
        sql_script = "SELECT SYMBOL, SNAME, VATURNOVER, PCHG, MCAP, TCAP FROM finchina.CHDQUOTE WHERE" \
                     " TDATE = {}".format(self.trade_date)
        data = pd.DataFrame(hbs.db_data_query('readonly', sql_script, page_size=5000)['data'])
        data = data[data['SYMBOL'].str[0].isin(['0', '3', '6'])]
        data = data[data['VATURNOVER'] > 1e-8]
        data = data[~data['SNAME'].str.contains('ST')]
        data = data[~data['SNAME'].str.contains('N')]
        data = data[~data['SNAME'].str.contains('C')]
        # 剔除新股
        list_status = pd.read_csv('D:\\kevin\\risk_model_jy\\RiskModel\\data\\common_data\\list_status.csv',
                                  index_col=0, dtype={"listDate": str})
        list_status['ticker'] = list_status['ticker'].apply(lambda x: str(x).zfill(6))
        trade_dt = datetime.datetime.strptime(self.trade_date, '%Y%m%d')
        pre_date = (trade_dt - datetime.timedelta(days=90)).strftime('%Y%m%d')
        list_status = list_status[list_status['listDate'] <= pre_date]
        data = data[data['SYMBOL'].isin(list_status['ticker'].tolist())]
        # data = data[data['PCHG'].abs() <= 10.1]
        market_df = data.rename(
            columns={"SYMBOL": "ticker", 'PCHG': "return", "TCAP": "marketValue", "MCAP": "floatM"})[
            ['ticker', 'return', 'marketValue', 'floatM']].dropna()

        shift_date = self._load_shift_date(self.trade_date)
        benchmark_cp = self._load_benchmark_components(shift_date)

        market_df = pd.merge(market_df, benchmark_cp, on='ticker', how='left')
        market_df['benchmark_id'].fillna('other', inplace=True)

        # 指数收益率
        sql_script = "SELECT JYRQ as TRADEDATE, ZQMC as INDEXNAME, SPJG as TCLOSE from funddb.ZSJY WHERE ZQDM = '{}' " \
                     "and JYRQ >= {} and JYRQ <= {}".format('000905', pre_date, self.trade_date)
        data = pd.DataFrame(hbs.db_data_query('readonly', sql_script)['data'])
        data['index_return'] = data['TCLOSE'].pct_change()
        index_return = data.set_index('TRADEDATE').loc[self.trade_date, 'index_return']

        self.market_df = market_df
        self.index_return = index_return

    @staticmethod
    def _cs_calculate(df):
        ret_series = df['return']
        return [ret_series.std(), ret_series.skew(), ret_series.kurt()]

    def calculate_cross_section_vol(self):
        market_df = self.market_df.copy()
        cs_vol = pd.DataFrame(columns=['std', 'skew', 'kurt'])
        # universe cut
        cut_dict = {
            "300": ["000300"],
            "500": ["000905"],
            "1000": ["000852"],
            "other": ["other"],
            "800": ["000300", "000905"],
            "not_800": ["000852", "other"],
            "1800": ["000300", "000905", "000852"],
            "not_300": ["000905", "000852", "other"],
            "Market_A": ["000300", "000905", "000852", "other"]
        }
        for key, value in cut_dict.items():
            tmp = market_df[market_df['benchmark_id'].isin(value)]
            cs_vol.loc[key] = self._cs_calculate(tmp)
        cs_vol.index.name = 'universe'
        cs_vol = pd.melt(cs_vol.reset_index(), id_vars=['universe'], var_name='type', value_name='value')
        cs_vol['factor_name'] = cs_vol['type'] + '_' + cs_vol['universe']
        cs_vol = cs_vol.set_index('factor_name')['value'].to_frame(self.trade_date)

        return cs_vol

    @staticmethod
    def _calculate(df, index_return):
        win_ratio = 100 - stats.percentileofscore(np.array(df['return']), index_return)
        win_med = df[df['return'] >= index_return]['return'].median()
        lose_med = df[df['return'] < index_return]['return'].median()

        return win_ratio, win_med, lose_med

    def calculate_win_ratio_factor(self):
        market_df = self.market_df.copy()
        index_return = self.index_return * 100.

        # 300 + 500
        win_ratio_df = pd.DataFrame(
            index=[self.trade_date], columns=['win_ratio_800', 'win_spread_800', 'lose_spread_800'])
        tmp_df = market_df[market_df['benchmark_id'].isin(['000300', '000905'])]
        win_ratio, win_med, lose_med = self._calculate(tmp_df, index_return)
        win_ratio_df.loc[self.trade_date] = [win_ratio, win_med - index_return, index_return - lose_med]
        df1 = win_ratio_df.copy()
        # 300 + 500 + 1000
        win_ratio_df = pd.DataFrame(
            index=[self.trade_date], columns=['win_ratio_1800', 'win_spread_1800', 'lose_spread_1800'])
        tmp_df = market_df[~market_df['benchmark_id'].isin(['other'])]
        win_ratio, win_med, lose_med = self._calculate(tmp_df, index_return)
        win_ratio_df.loc[self.trade_date] = [win_ratio, win_med - index_return, index_return - lose_med]
        df2 = win_ratio_df.copy()
        # all market
        win_ratio_df = pd.DataFrame(
            index=[self.trade_date], columns=['win_ratio_all', 'win_spread_all', 'lose_spread_all'])
        win_ratio, win_med, lose_med = self._calculate(market_df, index_return)
        win_ratio_df.loc[self.trade_date] = [win_ratio, win_med - index_return, index_return - lose_med]
        df3 = win_ratio_df.copy()

        win_ratio_df = pd.concat([df1, df2, df3], axis=1).T

        return win_ratio_df


class IndustryCr:
    def __init__(self, start_date, end_date):
        self.start_date = start_date
        self.end_date = end_date

    def _load_industry_data(self):
        start_dt = datetime.datetime.strptime(self.start_date, '%Y%m%d')
        pre_date = (start_dt - datetime.timedelta(days=50)).strftime('%Y%m%d')

        sql_script = "SELECT * FROM st_market.t_st_zs_hyzsdmdyb where fljb = {} and hyhfbz = 2".format('2')
        res = hbs.db_data_query('alluser', sql_script, page_size=5000)
        data = pd.DataFrame(res['data']).rename(columns={"zsdm": "SYMBOL", "flmc": "INDEXSNAME"})
        map_dict = data.set_index('SYMBOL')['INDEXSNAME'].to_dict()
        # 二级行业的微调
        map_dict.pop('399707')
        map_dict['801193'] = "券商"
        industry_data = []
        for key, value in map_dict.items():
            sql_script = "SELECT JYRQ as TRADEDATE, ZQMC as INDEXNAME, SPJG as TCLOSE, ZSZ as MKV " \
                         "FROM funddb.ZSJY WHERE " \
                         "ZQDM = '{}' and JYRQ >= {} and " \
                         "JYRQ <= {}".format(key, pre_date, self.end_date)
            res = hbs.db_data_query('readonly', sql_script, page_size=5000)
            df = pd.DataFrame(res['data'])
            df['INDEXNAME'] = value
            industry_data.append(df)
        industry_data = pd.concat(industry_data)

        industry_index = pd.pivot_table(
            industry_data, index='TRADEDATE', columns='INDEXNAME', values='TCLOSE').sort_index()
        industry_return = industry_index.pct_change().dropna().loc[self.start_date:]

        industry_mkv = pd.pivot_table(
            industry_data, index='TRADEDATE', columns='INDEXNAME', values='MKV').sort_index()
        industry_mkv = industry_mkv.reindex(industry_return.index).fillna(method='ffill')

        return industry_return, industry_mkv

    def _load_benchmark_return(self):
        start_dt = datetime.datetime.strptime(self.start_date, '%Y%m%d')
        pre_date = (start_dt - datetime.timedelta(days=50)).strftime('%Y%m%d')
        sql_script = "SELECT JYRQ as TRADEDATE, ZQMC as INDEXNAME, SPJG as TCLOSE from funddb.ZSJY WHERE ZQDM = '{}' " \
                     "and JYRQ >= {} and JYRQ <= {}".format('000905', pre_date, self.end_date)
        data = pd.DataFrame(hbs.db_data_query('readonly', sql_script)['data'])
        data['index_return'] = data['TCLOSE'].pct_change()
        index_return = data.set_index('TRADEDATE').loc[self.start_date:, 'index_return']

        return index_return

    def calculate(self):
        industry_return, industry_mkv = self._load_industry_data()
        index_return = self._load_benchmark_return()

        industry_factor = pd.DataFrame(index=industry_return.index)

        # 行业绝对胜率
        win_ratio_abs = industry_return.gt(0).sum(axis=1) / industry_return.shape[1]
        industry_factor['win_ratio_abs'] = win_ratio_abs
        # 行业相对中证500胜率
        win_ratio_relative = (industry_return.sub(index_return, axis=0)).gt(0).sum(axis=1) / industry_return.shape[1]
        industry_factor['win_ratio_relative'] = win_ratio_relative
        # 行业收益的波动率
        industry_factor['ind_std'] = industry_return.std(axis=1)
        # 行业收益的偏度
        industry_factor['ind_skew'] = industry_return.skew(axis=1)
        # 峰度
        industry_factor['ind_kurt'] = industry_return.kurt(axis=1)
        # 行业收益贡献: 标准差/偏度/峰度/spread
        assert (industry_return.shape == industry_mkv.shape)
        industry_ratio = industry_mkv.divide(industry_mkv.sum(axis=1), axis=0)
        industry_attr = industry_return.multiply(industry_ratio)
        industry_factor['ind_attr_std'] = industry_attr.std(axis=1)
        industry_factor['ind_attr_skew'] = industry_attr.skew(axis=1)
        industry_factor['ind_attr_kurt'] = industry_attr.kurt(axis=1)

        industry_factor['ind_spread'] = industry_return.apply(
            lambda x: sm.OLS(x.sort_values().values, sm.add_constant(
                np.arange(industry_return.shape[1]))).fit().params[1], axis=1)

        return industry_factor


class MarketStructure:
    def __init__(self, trade_date, benchmark_id):
        self.trade_date = trade_date
        self.benchmark_id = benchmark_id
        self._load_data()

    def _load_market_info(self):
        # A股行情
        sql_script = "SELECT SYMBOL, SNAME, VOTURNOVER, PCHG, MCAP, TCAP FROM finchina.CHDQUOTE WHERE" \
                     " TDATE = {}".format(self.trade_date)
        data = pd.DataFrame(hbs.db_data_query('readonly', sql_script, page_size=5000)['data'])
        data = data[data['SYMBOL'].str[0].isin(['0', '3', '6'])]
        data = data[data['VOTURNOVER'] > 1e-8]
        data = data[~data['SNAME'].str.contains('ST')]
        data = data[~data['SNAME'].str.contains('N')]
        data = data[~data['SNAME'].str.contains('C')]
        # 剔除新股
        list_status = pd.read_csv('D:\\kevin\\risk_model_jy\\RiskModel\\data\\common_data\\list_status.csv',
                                  index_col=0, dtype={"listDate": str})
        list_status['ticker'] = list_status['ticker'].apply(lambda x: str(x).zfill(6))
        trade_dt = datetime.datetime.strptime(self.trade_date, '%Y%m%d')
        pre_date = (trade_dt - datetime.timedelta(days=90)).strftime('%Y%m%d')
        list_status = list_status[list_status['listDate'] <= pre_date]
        data = data[data['SYMBOL'].isin(list_status['ticker'].tolist())]

        data = data[data['PCHG'].abs() <= 10.1]

        market_df = data.rename(
            columns={"SYMBOL": "ticker", 'PCHG': "return", "TCAP": "marketValue", "MCAP": "floatM"})[
            ['ticker', 'return', 'marketValue', 'floatM']].dropna()
        # 指数收益率
        sql_script = "SELECT JYRQ as TRADEDATE, ZQMC as INDEXNAME, SPJG as TCLOSE from funddb.ZSJY WHERE ZQDM = '{}' " \
                     "and JYRQ >= {} and JYRQ <= {}".format(self.benchmark_id, pre_date, self.trade_date)
        data = pd.DataFrame(hbs.db_data_query('readonly', sql_script)['data'])
        data['index_return'] = data['TCLOSE'].pct_change()
        index_return = data.set_index('TRADEDATE').loc[self.trade_date, 'index_return']

        return market_df, index_return

    def _load_style_factor_return(self):
        sql_script = "SELECT * FROM st_ashare.r_st_barra_factor_return where TRADE_DATE = '{}'".format(self.trade_date)
        data = pd.DataFrame(hbs.db_data_query('alluser', sql_script)['data'])
        size_return = data.set_index('factor_name').loc['size', 'factor_ret']

        return size_return

    def _load_industry_data(self):
        start_dt = datetime.datetime.strptime(self.trade_date, '%Y%m%d')
        pre_date = (start_dt - datetime.timedelta(days=50)).strftime('%Y%m%d')

        sql_script = "SELECT * FROM st_market.t_st_zs_hyzsdmdyb where fljb = {} and hyhfbz = 2".format('1')
        res = hbs.db_data_query('alluser', sql_script, page_size=5000)
        data = pd.DataFrame(res['data']).rename(columns={"zsdm": "SYMBOL", "flmc": "INDEXSNAME"})
        map_dict = data.set_index('SYMBOL')['INDEXSNAME'].to_dict()
        industry_index = []
        for key, value in map_dict.items():
            sql_script = "SELECT JYRQ as TRADEDATE, ZQMC as INDEXNAME, SPJG as TCLOSE, LTSZ as NEG_MKV " \
                         "FROM funddb.ZSJY WHERE " \
                         "ZQDM = '{}' and JYRQ >= {} and " \
                         "JYRQ <= {}".format(key, pre_date, self.trade_date)
            res = hbs.db_data_query('readonly', sql_script, page_size=5000)
            df = pd.DataFrame(res['data'])
            df['INDEXNAME'] = value
            industry_index.append(df)
        industry_index = pd.concat(industry_index)

        return industry_index

    def _load_data(self):
        market_df, index_return = self._load_market_info()
        size_return = self._load_style_factor_return()
        industry_index = self._load_industry_data()

        self.data_param = {
            "market_df": market_df,
            "index_return": index_return,
            "size_return": size_return,
            "industry_index": industry_index
        }

    def get_construct_result(self):
        data_param = self.data_param.copy()

        structure_df = pd.DataFrame(index=[self.trade_date], columns=[
            'cs_vol', 'cs_vol_w', 'skew', 'kurt', 'win_ratio', 'win_med', 'lose_med', 'size_return', 'ind_cr'])

        market_df = data_param['market_df']
        index_return = data_param['index_return'] * 100.
        structure_df.loc[self.trade_date, 'cs_vol'] = market_df['return'].std() / 100.
        structure_df.loc[self.trade_date, 'cs_vol_w'] = \
            DescrStatsW(np.array(market_df['return']), weights=np.array(market_df['floatM']), ddof=1).std / 100.
        structure_df.loc[self.trade_date, 'skew'] = market_df['return'].skew()
        structure_df.loc[self.trade_date, 'kurt'] = market_df['return'].kurt()
        win_ratio = round(100 - stats.percentileofscore(np.array(market_df['return']), index_return), 1)
        win_med = market_df[market_df['return'] >= index_return]['return'].median() - index_return
        lose_med = market_df[market_df['return'] < index_return]['return'].median() - index_return
        structure_df.loc[self.trade_date, 'win_ratio'] = win_ratio
        structure_df.loc[self.trade_date, 'win_med'] = win_med
        structure_df.loc[self.trade_date, 'lose_med'] = lose_med
        size_return = data_param['size_return']
        structure_df.loc[self.trade_date, 'size_return'] = size_return

        industry_index = data_param['industry_index']
        tmp = pd.pivot_table(
            industry_index, index='TRADEDATE', columns='INDEXNAME', values='TCLOSE').sort_index()
        industry_return = tmp.pct_change().dropna().loc[self.trade_date]
        structure_df.loc[self.trade_date, 'ind_cr'] = industry_return.gt(0).sum() / len(industry_return)

        return structure_df


alpha_dict = {
    "衍复指增三号": "SJH866",
    "九坤日享中证500指数增强1号": "ST9804",
    "因诺聚配中证500指数增强": "SGX346",
    "启林中证500指数增强1号": "SGY379"
}


if __name__ == '__main__':
    # MarketSpread('20211117').calculate_cross_section_vol()
    # sdate = '20180101'
    sdate = '20211101'
    edate = '20211112'

    # from tqdm import tqdm
    # date_list = get_trading_day_list(sdate, edate)
    # cs_vol_list = []
    # win_ratio_list = []
    # for t_date in tqdm(date_list):
    #     market_spread = MarketSpread(t_date)
    #     res1 = market_spread.calculate_cross_section_vol()
    #     res2 = market_spread.calculate_win_ratio_factor()
    #     cs_vol_list.append(res1)
    #     win_ratio_list.append(res2)
    #
    # factor1 = pd.concat(cs_vol_list, axis=1).T
    # factor2 = pd.concat(win_ratio_list, axis=1).T
    #
    # factor1.to_csv('D:\\市场微观结构图\\market_factor_save\\cs_vol.csv')
    # factor2.to_csv('D:\\市场微观结构图\\market_factor_save\\win_ratio.csv')

    # ind_factor = IndustryCr(sdate, edate).calculate()
    # ind_factor.to_csv('D:\\市场微观结构图\\market_factor_save\\ind_cr.csv')

    # market_structure = MarketStructure()
    MarketHist('20220429', '000905').daily_plot()


"""
信用类指标
"""
import pandas as pd
from datetime import datetime
from hbshare.quant.Kevin.asset_allocation.macro_index.util import create_table, delete_duplicate_records, WriteToDB
from WindPy import w

w.start()


class Credit:
    def __init__(self, start_date, end_date, is_increment=1):
        self.start_date = start_date
        self.end_date = end_date
        self.is_increment = is_increment
        self.table_name = 'mac_credit'

    def get_credit_data(self):
        """
        信用类数据：信用利差(余额加权):产业债AAA、信用利差(余额加权):产业债AA+、信用利差(余额加权):产业债AA、
                 信用利差(余额加权):城投债AAA、信用利差(余额加权):城投债AA+、信用利差(余额加权):城投债AA、
                 实体经济部门杠杆率、居民部门杠杆率、非金融企业部门杠杆率、政府部门杠杆率、商业银行:杠杆率
        """
        index_list = ['M1010107', 'M1010108', 'M1010109', 'M1010358', 'M1010359', 'M1010360',
                      'M6404532', 'M6404533', 'M6404534', 'M6404535', 'M6194530']
        name_dict = {'M1010107': "credit_spread_indu_AAA", "M1010108": "credit_spread_indu_AA_plus",
                     "M1010109": "credit_spread_indu_AA", "M1010358": "credit_spread_urban_AAA",
                     "M1010359": "credit_spread_urban_AA_plus", "M1010360": "credit_spread_urban_AA",
                     "M6404532": "leverage_1", "M6404533": "leverage_2", "M6404534": "leverage_3",
                     "M6404535": "leverage_4", "M6194530": "leverage_5"}

        res = w.edb(','.join(index_list), self.start_date, self.end_date)
        if res.ErrorCode != 0:
            data = pd.DataFrame()
            print("fetch credit data error: start_date = {}, end_date = {}".format(self.start_date, self.end_date))
        else:
            if len(res.Data) == 1:
                data = pd.DataFrame(res.Data[0], index=res.Codes, columns=res.Times).T
            else:
                data = pd.DataFrame(res.Data, index=res.Codes, columns=res.Times).T
            data.index.name = 'trade_date'
            data.reset_index(inplace=True)
            data['trade_date'] = data['trade_date'].apply(lambda x: datetime.strftime(x, '%Y%m%d'))
            data.rename(columns=name_dict, inplace=True)

        return data

    def get_construct_result(self):
        if self.is_increment == 1:
            data = self.get_credit_data()
            sql_script = "delete from {} where trade_date in ({})".format(
                self.table_name, ','.join(data['trade_date'].tolist()))
            # delete first
            delete_duplicate_records(sql_script)
            # add new records
            WriteToDB().write_to_db(data, self.table_name)
        else:
            sql_script = """
                create table mac_credit(
                id int auto_increment primary key,
                trade_date date not null unique,
                credit_spread_indu_AAA decimal(6, 2),
                credit_spread_indu_AA_plus decimal(6, 2),
                credit_spread_indu_AA decimal(6, 2),
                credit_spread_urban_AAA decimal(6, 2),
                credit_spread_urban_AA_plus decimal(6, 2),
                credit_spread_urban_AA decimal(6, 2),                
                leverage_1 decimal(6, 2),
                leverage_2 decimal(6, 2),
                leverage_3 decimal(6, 2),
                leverage_4 decimal(6, 2),
                leverage_5 decimal(6, 2)) 
            """
            create_table(self.table_name, sql_script)
            data = self.get_credit_data()
            WriteToDB().write_to_db(data, self.table_name)


if __name__ == '__main__':
    Credit('2005-12-01', '2021-04-23', is_increment=0).get_construct_result()


# -*- coding: utf-8 -*-

from hbshare.fe.xwq.analysis.orm.config import Config
from hbshare.fe.xwq.analysis.utils.singleton import singleton
from logging import config as configs
import logging


@singleton
class LoggerUtils:
    def __init__(self):
        self.logger = logging.getLogger()
        configs.fileConfig(Config().get_logger_properties())

    def get_logger(self):
        return self.logger


"""
股票指数截面波动率指标
"""
import pandas as pd
from datetime import datetime, timedelta
import requests
import json
from hbshare.quant.Kevin.asset_allocation.macro_index.util import create_table, delete_duplicate_records, WriteToDB


class StockCrossSectionVol:
    def __init__(self, start_date, end_date, is_increment=1):
        self.start_date = start_date
        self.end_date = end_date
        self.is_increment = is_increment
        self.table_name = 'mac_stock_cross_section_vol'
        self._init_api_params()

    def _init_api_params(self):
        self.url = 'http://fdc-query.intelnal.howbuy.com/query/data/commonapi?dataTrack=xxxxx'
        self.headers = {'Content-Type': 'application/json'}
        self.post_body = {"database": "readonly"}

    def fetch_data_batch(self, sql_script):
        post_body = self.post_body.copy()
        post_body['sql'] = sql_script
        post_body["ifByPage"] = False
        res = requests.post(url=self.url, data=json.dumps(post_body), headers=self.headers).json()
        n = res['pages']
        all_data = []
        for i in range(1, n + 1):
            post_body["ifByPage"] = True
            post_body['pageNum'] = i
            res = requests.post(url=self.url, data=json.dumps(post_body), headers=self.headers).json()
            all_data.append(pd.DataFrame(res['data']))
        all_data = pd.concat(all_data)

        return all_data

    def get_cross_section_vol(self):
        pre_dt = datetime.strptime(self.start_date, '%Y-%m-%d') - timedelta(days=60)
        pre_date = pre_dt.strftime('%Y%m%d')
        end_date = self.end_date.replace('-', '')

        sql_script = "SELECT JYRQ, SFJJ, SFZM, SFYM FROM funddb.JYRL WHERE JYRQ >= {} and JYRQ <= {}".format(
            pre_date, end_date)
        post_body = self.post_body
        post_body['sql'] = sql_script
        res = requests.post(url=self.url, data=json.dumps(post_body), headers=self.headers).json()
        df = pd.DataFrame(res['data']).rename(
            columns={"JYRQ": 'calendarDate', "SFJJ": 'isOpen',
                     "SFZM": "isWeekEnd", "SFYM": "isMonthEnd"}).sort_values(by='calendarDate')
        df['isOpen'] = df['isOpen'].astype(int).replace({0: 1, 1: 0})
        df['isWeekEnd'] = df['isWeekEnd'].fillna(0).astype(int)
        df['isMonthEnd'] = df['isMonthEnd'].fillna(0).astype(int)
        trading_day_list = df[df['isOpen'] == 1]['calendarDate'].tolist()
        actual_trading_day_list = [x for x in trading_day_list if x >= self.start_date.replace('-', '')]
        month_end_list = df[df['isMonthEnd'] == 1]['calendarDate'].tolist()
        # 指数成分
        period_bm_cons = []
        index_list = ['000300', '000905', '000852']
        for date in month_end_list:
            bm_cons = []
            for benchmark_id in index_list:
                sql_script = "SELECT a.EndDate, a.Weight, c.SecuCode " \
                             "FROM hsjy_gg.LC_IndexComponentsWeight a, hsjy_gg.SecuMain b, hsjy_gg.SecuMain c " \
                             "WHERE a.indexCode = b.innerCode and b.SecuCode = '{}' and " \
                             "to_char(a.EndDate, 'yyyymmdd') = '{}' and b.SecuCategory = 4 " \
                             "and a.InnerCode = c.InnerCode".format(benchmark_id, date)
                post_body = self.post_body
                post_body['sql'] = sql_script
                res = requests.post(url=self.url, data=json.dumps(post_body), headers=self.headers).json()
                if pd.DataFrame(res['data']).empty:
                    weight_df = pd.DataFrame(columns=['SECUCODE', 'trade_date', 'benchmark_id'])
                else:
                    weight_df = pd.DataFrame(res['data'])[['SECUCODE']]
                    weight_df['trade_date'] = date
                    weight_df['benchmark_id'] = benchmark_id
                bm_cons.append(weight_df)
            bm_cons = pd.concat(bm_cons)
            period_bm_cons.append(bm_cons)
        period_bm_cons = pd.concat(period_bm_cons)
        period_bm_cons['benchmark_id'] = period_bm_cons['benchmark_id'].replace({"000300": 1, "000905": 2, "000852": 3})
        pivot_df = pd.pivot_table(
            period_bm_cons, index='trade_date', columns='SECUCODE', values='benchmark_id').fillna(4.).sort_index()
        bm_df = pivot_df.reindex(trading_day_list).fillna(method='ffill').reindex(actual_trading_day_list)
        # 行情
        sql_script = "SELECT SYMBOL, TDATE, PCHG, VOTURNOVER FROM finchina.CHDQUOTE WHERE" \
                     " TDATE >= {} and TDATE <= {}".format(actual_trading_day_list[0], actual_trading_day_list[-1])
        mkt_info = self.fetch_data_batch(sql_script)
        mkt_info = mkt_info[(mkt_info['VOTURNOVER'] > 0) & (mkt_info['PCHG'].abs() <= 20)]
        mkt_info['TDATE'] = mkt_info['TDATE'].map(str)
        res = []
        for date in actual_trading_day_list:
            tdf = mkt_info[mkt_info['TDATE'] == date].set_index('SYMBOL')['PCHG'] / 100.
            tdf = tdf.to_frame('ret').merge(
                bm_df.loc[date].to_frame('sign'), left_index=True, right_index=True, how='left').fillna(4.)
            cs_vol = tdf.groupby('sign')['ret'].std().to_frame(date).T
            res.append(cs_vol)

        res = pd.concat(res).rename(columns={1.0: "HS300", 2.0: "ZZ500", 3.0: "ZZ1000", 4.0: "OTHER"})
        res.index.name = 'trade_date'
        res.reset_index(inplace=True)

        return res

    def get_construct_result(self):
        if self.is_increment == 1:
            data = self.get_cross_section_vol()
            sql_script = "delete from {} where trade_date in ({})".format(
                self.table_name, ','.join(data['trade_date'].tolist()))
            # delete first
            delete_duplicate_records(sql_script)
            # add new records
            WriteToDB().write_to_db(data, self.table_name)
        else:
            sql_script = """
                create table mac_stock_cross_section_vol(
                id int auto_increment primary key,
                trade_date date not null unique,
                HS300 decimal(5, 4),
                ZZ500 decimal(5, 4),
                ZZ1000 decimal(5, 4),
                OTHER decimal(5, 4)) 
            """
            create_table(self.table_name, sql_script)
            data = self.get_cross_section_vol()
            WriteToDB().write_to_db(data, self.table_name)


if __name__ == '__main__':
    StockCrossSectionVol('2021-09-01', '2021-10-28', is_increment=1).get_construct_result()


"""
私募基金估值表持仓入库
"""
import os
import pandas as pd
from hbshare.quant.Kevin.asset_allocation.macro_index.util import create_table, delete_duplicate_records, WriteToDB


class HoldingExtractor:
    def __init__(self, data_path, table_name, fund_name, is_increment=1):
        self.data_path = data_path
        self.table_name = table_name
        self.fund_name = fund_name
        self.is_increment = is_increment

    def _load_portfolio_weight(self):
        filenames = os.listdir(self.data_path)
        filenames = [x for x in filenames if x.split('.')[-1] in ['xls', 'xlsx']]

        portfolio_weight_list = []
        for name in filenames:
            date = name.split('_')[-2]
            # 国君 & 海通
            if self.fund_name in ['因诺聚配中证500指数增强', '凡二中证500增强9号1期', '赫富500指数增强一号', '赫富1000指数增强一号',
                                  '宽德金选中证500指数增强6号', 'TEST', '量锐62号', '启林广进中证1000指数增强',
                                  '白鹭精选量化鲲鹏十号', '伯兄建康', '伯兄熙宁', '世纪前沿指数增强2号',
                                  '云起量化指数增强1号', '罗维盈安凌云增强2号']:
                context = "市值占净值%"
                if self.fund_name in ['伯兄建康', '伯兄熙宁']:
                    hd = 2
                    context = "市值占净值(%)"
                elif self.fund_name in ['世纪前沿指数增强2号']:
                    hd = 2 if date >= '20210630' else 0
                    context = "市值占净值(%)"
                else:
                    hd = 3
                data = pd.read_excel(
                    os.path.join(self.data_path, name), sheet_name=0, header=hd).dropna(subset=['科目代码'])
                sh = data[data['科目代码'].str.startswith('11020101')]
                sz = data[data['科目代码'].str.startswith('11023101')]
                cyb = data[data['科目代码'].str.startswith('11024101')]
                kcb = data[data['科目代码'].str.startswith('1102C101')]
                df = pd.concat([sh, sz, cyb, kcb], axis=0).dropna()
                df['ticker'] = df['科目代码'].apply(lambda x: x[-6:])
                df.rename(columns={"科目名称": "sec_name", context: "weight"}, inplace=True)
                if self.fund_name in ['凡二中证500增强9号1期', '量锐62号', '罗维盈安凌云增强2号'] and \
                        type(df['weight'].tolist()[0]) == str:
                    df['weight'] = df['weight'].str.strip('%').astype(float)
            # 招商
            elif self.fund_name in ['星阔广厦1号中证500指数增强', '星阔上林1号', '星阔山海6号',
                                    '量客卓宇六号', '乾象中证500指数增强1号', '水木博雅500指增',
                                    '概率1000指增1号', '概率500指增2号', '稳博中性稳稳系列1号', '稳博中性稳稳系列2号',
                                    '顽岩中证500指数增强1号']:
                header = 6 if self.fund_name in ['星阔上林1号', '水木博雅500指增', '乾象中证500指数增强1号',
                                                 '稳博中性稳稳系列1号', '稳博中性稳稳系列2号'] else 7
                data = pd.read_excel(
                    os.path.join(self.data_path, name), sheet_name=0, header=header).dropna(subset=['科目代码'])
                sh = data[data['科目代码'].str.endswith('SH')]
                sz = data[data['科目代码'].str.endswith('SZ')]
                df = pd.concat([sh, sz], axis=0)
                df['ticker'] = df['科目代码'].apply(lambda x: x.split(' ')[0][-6:])
                df.rename(columns={"科目名称": "sec_name", "市值占比": "weight"}, inplace=True)
                if self.fund_name == "乾象中证500指数增强1号":
                    # 处理一下分红
                    ratio = data[data['科目代码'] == '资产净值']['市值占比'].values[0] / \
                            data[data['科目代码'] == '资产合计']['市值占比'].values[0]
                    df['weight'] *= ratio
                df['weight'] *= 100.
            # 海通但有信用账户
            elif self.fund_name in ['朋锦金石炽阳']:
                data = pd.read_excel(
                    os.path.join(self.data_path, name), sheet_name=0, header=3).dropna(subset=['科目代码'])
                sh1 = data[data['科目代码'].str.startswith('11020101')]
                sh2 = data[data['科目代码'].str.startswith('11021101')]
                sz1 = data[data['科目代码'].str.startswith('11023101')]
                sz2 = data[data['科目代码'].str.startswith('11023201')]
                cyb1 = data[data['科目代码'].str.startswith('11024101')]
                cyb2 = data[data['科目代码'].str.startswith('11021501')]
                kcb1 = data[data['科目代码'].str.startswith('1102C101')]
                kcb2 = data[data['科目代码'].str.startswith('1102D201')]
                df = pd.concat([sh1, sh2, sz1, sz2, cyb1, cyb2, kcb1, kcb2], axis=0).dropna()
                df['ticker'] = df['科目代码'].apply(lambda x: x[-6:])
                df.rename(columns={"科目名称": "sec_name", "市值占净值%": "weight"}, inplace=True)
                df['weight'] = 100. * df['weight'] / df['weight'].sum()
            else:
                date = None
                df = pd.DataFrame()

            df['trade_date'] = date
            portfolio_weight_list.append(df[['trade_date', 'ticker', 'sec_name', 'weight']])

        portfolio_weight_df = pd.concat(portfolio_weight_list)
        portfolio_weight_df = portfolio_weight_df.groupby(
            ['trade_date', 'ticker', 'sec_name'])['weight'].sum().reset_index()
        portfolio_weight_df['fund_name'] = self.fund_name

        # print(portfolio_weight_df.groupby('trade_date')['weight'].sum().sort_index())

        return portfolio_weight_df

    def writeToDB(self):
        if self.is_increment == 1:
            data = self._load_portfolio_weight()
            trading_day_list = data['trade_date'].unique().tolist()
            sql_script = "delete from {} where trade_date in ({}) and fund_name = '{}'".format(
                self.table_name, ','.join(trading_day_list), self.fund_name)
            # delete first
            delete_duplicate_records(sql_script)
            # add new records
            WriteToDB().write_to_db(data, self.table_name)
        else:
            sql_script = """
                create table {}(
                id int auto_increment primary key,
                trade_date date not null,
                ticker varchar(10),
                sec_name varchar(20),
                weight decimal(5, 4),
                fund_name varchar(20))
            """.format(self.table_name)
            create_table(self.table_name, sql_script)
            data = self._load_portfolio_weight()
            WriteToDB().write_to_db(data, self.table_name)


if __name__ == '__main__':
    HoldingExtractor(data_path='D:\\估值表基地\\顽岩中证500指数增强1号', table_name="private_fund_holding",
                     fund_name="顽岩中证500指数增强1号").writeToDB()



"""
可转债定价模型；二叉树模型
"""
import numpy as np
# from cb_pricing_BS import run_time
from hbshare.quant.Kevin.convertible_bond.cb_pricing_BS import _cashFlowCalc


def _cashFlowDict(termCoupon, strMaturityDate, strNowDate):
    cashFlow, cashFlowTime = _cashFlowCalc(termCoupon, strMaturityDate, strNowDate)
    dictCF = {}
    for i, cash in enumerate(cashFlow):
        dictCF[round(cashFlowTime[i] * 250)] = cash

    return dictCF


def cashFlowGenerator(dictCF, tDate):
    return dictCF[tDate] if tDate in dictCF else 0


def _underlyingTree(s, u, d, numNodes):
    """
    生成股价树
    """
    arrTree = np.zeros([numNodes, numNodes])
    arrTree[0, 0] = s

    for i in range(numNodes):
        for j in range(i + 1):
            arrTree[j, i] = s * (u ** (i - j)) * (d ** j)

    return arrTree


def _calcOptionTree(dictCF, arrTree, x, p_u, p_d, r, convertStart=5.5):
    """
    在已有树的情况下，倒算转债价值
    """
    numNodes = int(max(dictCF))

    arrVal = np.zeros_like(arrTree)
    arrVal[:, -1] = list(map(lambda y: max([x, y]), arrTree[:, -1]))

    r = np.log(1 + r)

    for i in range(numNodes - 2, -1, -1):
        cf = cashFlowGenerator(dictCF, i)

        for j in range(i + 1):
            if i / 250.0 <= convertStart:
                arrVal[j, i] = max(
                    [np.exp(-r / 250.0) * (p_u * arrVal[j, i + 1] + p_d * arrVal[j + 1, i + 1]) + cf, arrTree[j, i]])
            else:
                arrTree[j, i] = np.exp(-r / 250.0) * (p_u * arrVal[j, i + 1] + p_d * arrVal[j + 1, i + 1]) + cf

    return arrVal


def binomialTree(dictCF, s, x, vol, r, convertStart=5.5):
    """
    利用二叉树倒算转债价值
    @param dictCF: 现金流，票息
    @param s: 标的股票的现价
    @param x: 执行价格/转股价格
    @param vol: 股票的标准差
    @param r: 无风险利率
    @param convertStart: 期限
    @return:
    """
    numNodes = max(dictCF)
    u = np.exp(vol * np.sqrt(1 / 250.))
    d = 1 / u
    arrTree = _underlyingTree(s, u, d, int(numNodes))

    rc = np.log(1 + r)
    p_u = (np.exp(rc / 250.0) - d) / (u - d)
    p_d = 1 - p_u

    arrVal = _calcOptionTree(dictCF, arrTree, x, p_u, p_d, r, convertStart)

    return arrVal[0, 0]


# @run_time
def cbPricingTree(stock, term, now, vol, r, isPrint=True):
    """
    二叉树定价函数
    @param stock: 当前股价
    @param term: 转债条款变量
    @param now: 当前日期
    @param vol: 隐含波动率
    @param r: 无风险利率(同等级期限信用债收益率)
    @param isPrint
    """
    dictCF = _cashFlowDict(term['Coupon'], term['Maturity'], now)

    s = stock / term['ConvPrice'] * 100
    x = term['Coupon'][-1]
    convertstart = term['ConvertStart']

    ret = binomialTree(dictCF, s, x, vol, r, convertstart)

    if isPrint:
        print("二叉树模型估算的可转债价值: {}".format(ret, '0.2f'))

    return ret


if __name__ == '__main__':
    term_CB = {
        "ConvPrice": 100,
        "Maturity": "2022/3/10",
        "ConvertStart": 5.5,
        "Coupon": [0.3, 0.5, 0.8, 1.3, 1.5, 108],
        "Recall": [5.5, 15, 30, 130],
        "Resell": [2, 30, 30, 70, 103]
    }

    stock_price = 115.0
    stock_vol = 0.258672
    rate = 0.038

    now_date = '2019/6/16'

    cbPricingTree(stock_price, term_CB, now_date, stock_vol, rate)


#!/usr/bin/env python
# -*- coding:utf-8 -*-
# @FileName  :simu_index_test.py
# @Time      :2021/4/27 2:18 下午
# @Author    :meng.lv

import hbshare as hbs
import pandas  as pd
import time

if __name__ == "__main__":
    pd.set_option('display.max_columns', None)
    pd.set_option('display.unicode.east_asian_width', True)
    pd.set_option('display.width', 180)  # 设置打印宽度(**重要**)
    #data = hbs.simu_index("RISK_DOWNSIDE_WEEK","S00748", '20210423', ["2101"], show_log=False)

    #print(data)
    si = hbs.SimuIndex("RISK_MAX_DRAWDOWN_RANK_DAY",show_log=False)
    dt = si.get("S00748", '20210419', ["2101"])
    print(dt)
    cfg = si.get_config()
    indices = []
    for index, row in cfg.iterrows():
        if row['DAY']:
            indices.append((index+"_DAY","日频_"+row['NAME_CN']))
        if row['WEEK']:
            indices.append((index+"_WEEK","周频_"+row['NAME_CN']))
        if row['MONTH']:
            indices.append((index+"_MONTH","月频_"+row['NAME_CN']))
        if row['YEAR']:
            indices.append((index+"_YEAR","年频_"+row['NAME_CN']))
    print(indices)
    # sql = "select JZRQ,JJDM,HBDR,HB1Z,HB1Y,HB3Y,HB6Y from funddb.jjhb t where t.jjdm = '020005' and t.JZRQ = '20090526'"
    # data = hbs.db_data_query('readonly', sql)
    # print(data)

